@@ -1,11 +1,11 @@
-# $Id: FollowTail.pm,v 1.54 2005/04/12 23:02:18 rcaputo Exp $
+# $Id: FollowTail.pm 2160 2006-12-31 21:35:35Z rcaputo $
 
 package POE::Wheel::FollowTail;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.54 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my($r)=(q$Revision: 2160 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw( croak carp );
 use Symbol qw( gensym );
@@ -353,6 +353,24 @@ sub _define_timer_states {
                   if $new_stat[7] < $last_stat->[7];
               };
 
+              # The file may have rolled.  Try one more read before moving on.
+              if (
+                defined $$handle and
+                defined(my $raw_input = $driver->get($$handle))
+              ) {
+
+                # First read the remainder of the file.
+                # Got input.  Read a bunch of it, then poll again right away.
+                if (@$raw_input) {
+                  TRACE_POLL and warn "<poll> " . time . " raw input\n";
+                  foreach my $cooked_input (@{$filter->get($raw_input)}) {
+                    TRACE_POLL and warn "<poll> " . time . " cooked input\n";
+                    $k->call($ses, $$event_input, $cooked_input, $unique_id);
+                  }
+                }
+                $k->yield($$state_read) if defined $$state_read;
+              }
+
               @$last_stat = @new_stat;
               close $$handle if defined $$handle;
               $$handle = _open_file($filename);
@@ -435,16 +453,6 @@ sub event {
       carp "ignoring unknown FollowTail parameter '$name'";
     }
   }
-
-  if ($self->[SELF_FOLLOW_MODE] & MODE_TIMER) {
-    $self->_define_timer_states();
-  }
-  elsif ($self->[SELF_FOLLOW_MODE] & MODE_SELECT) {
-    $self->_define_select_states();
-  }
-  else {
-    die;
-  }
 }
 
 #------------------------------------------------------------------------------
@@ -538,6 +546,16 @@ appended to its file.
 
 This is a read-only wheel so it does not include a put() method.
 
+=head1 CONSTRUCTOR
+
+=over 
+
+=item new
+
+new() creates a new wheel, returning the wheels reference.
+
+=back
+
 =head1 PUBLIC METHODS
 
 =over 2
@@ -724,16 +742,6 @@ the entire POE distribution.
 
 This wheel can't tail pipes and consoles on some systems.
 
-Because this wheel is cooperatively multitasked, it may lose records
-just prior to a file reset.  For a more robust way to watch files,
-consider using POE::Wheel::Run and your operating system's native
-"tail" utility instead.
-
-  $heap->{tail} = POE::Wheel::Run->new(
-    Program     => [ "/usr/bin/tail", "-f", $file_name ],
-    StdoutEvent => "log_record",
-  );
-
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
