@@ -1,11 +1,11 @@
-# $Id: Run.pm,v 1.67 2005/04/18 05:47:34 rcaputo Exp $
+# $Id: Run.pm 2179 2007-03-21 07:55:15Z rcaputo $
 
 package POE::Wheel::Run;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.67 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my($r)=(q$Revision: 2179 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(carp croak);
 use POSIX qw(
@@ -27,13 +27,13 @@ BEGIN {
   }
 
   if (POE::Kernel::RUNNING_IN_HELL) {
-      eval    { require Win32::Console; };
-      if ($@) { die "Win32::Console failed to load:\n$@" }
-      else    { Win32::Console->import(); };
+    eval    { require Win32::Console; };
+    if ($@) { die "Win32::Console failed to load:\n$@" }
+    else    { Win32::Console->import(); };
 
-      eval    { require Win32API::File; };
-      if ($@) { die "Win32API::File but failed to load:\n$@" }
-      else    { Win32API::File->import( qw(FdGetOsFHandle) ); };
+    eval    { require Win32API::File; };
+    if ($@) { die "Win32API::File but failed to load:\n$@" }
+    else    { Win32API::File->import( qw(FdGetOsFHandle) ); };
   }
 
   # How else can I get them out?!
@@ -148,14 +148,14 @@ sub new {
   my $conduit = delete $params{Conduit};
   my $stdio_type;
   if (defined $conduit) {
-    croak "$type\'s Conduit type ($conduit) is unknown"
-      if (
-        $conduit ne 'pipe' and
-        $conduit ne 'pty'  and
-        $conduit ne 'socketpair' and
-        $conduit ne 'inet'
-      );
-    unless ($conduit eq "pty") {
+    croak "$type\'s Conduit type ($conduit) is unknown" if (
+      $conduit ne 'pipe' and
+      $conduit ne 'pty'  and
+      $conduit ne 'pty-pipe' and
+      $conduit ne 'socketpair' and
+      $conduit ne 'inet'
+    );
+    unless ($conduit =~ /^pty(-pipe)?$/) {
       $stdio_type = $conduit;
       $conduit = "pipe";
     }
@@ -251,11 +251,17 @@ sub new {
   croak "could not create semaphore pipe: $!" unless defined $sem_pipe_read;
 
   # Use IO::Pty if requested.  IO::Pty turns on autoflush for us.
-  if ($conduit eq 'pty') {
+  if ($conduit =~ /^pty(-pipe)?$/) {
     croak "IO::Pty is not available" unless PTY_AVAILABLE;
 
     $stdin_write = $stdout_read = IO::Pty->new();
     croak "could not create master pty: $!" unless defined $stdout_read;
+
+    if ($conduit eq "pty-pipe") {
+      ($stderr_read, $stderr_write) = POE::Pipe::OneWay->new();
+      croak "could not make stderr pipes: $!"
+        unless defined $stderr_read and defined $stderr_write;
+    }
   }
 
   # Use pipes otherwise.
@@ -284,31 +290,33 @@ sub new {
   # Fork!  Woo-hoo!
   my $pid = fork;
 
-  # Stdio should not be tied.  Resolves rt.cpan.org ticket 1648.
-  if (tied *STDOUT) {
-    carp "Cannot redirect into tied STDOUT.  Untying it";
-    untie *STDOUT;
-  }
-  if (tied *STDERR) {
-    carp "Cannot redirect into tied STDERR.  Untying it";
-    untie *STDERR;
-  }
-
   # Child.  Parent side continues after this block.
   unless ($pid) {
     croak "couldn't fork: $!" unless defined $pid;
 
+    # Stdio should not be tied.  Resolves rt.cpan.org ticket 1648.
+    if (tied *STDOUT) {
+      carp "Cannot redirect into tied STDOUT.  Untying it";
+      untie *STDOUT;
+    }
+    if (tied *STDERR) {
+      carp "Cannot redirect into tied STDERR.  Untying it";
+      untie *STDERR;
+    }
+
     # If running pty, we delay the slave side creation 'til after
     # doing the necessary bits to become our own [unix] session.
-    if ($conduit eq 'pty') {
+    if ($conduit =~ /^pty(-pipe)?$/) {
 
       # Become a new unix session.
       # Program 19.3, APITUE.  W. Richard Stevens built my hot rod.
       eval 'setsid()' unless $no_setsid;
 
       # Open the slave side of the pty.
-      $stdin_read = $stdout_write = $stderr_write = $stdin_write->slave();
+      $stdin_read = $stdout_write = $stdin_write->slave();
       croak "could not create slave pty: $!" unless defined $stdin_read;
+      ## for a simple pty conduit, stderr is wedged into stdout:
+      $stderr_write = $stdout_write if $conduit eq 'pty';
 
       # Acquire a controlling terminal.  Program 19.3, APITUE.
       if (defined TIOCSCTTY and not defined CIBAUD) {
@@ -395,30 +403,27 @@ sub new {
     close $stdin_write;
     close $stdout_read;
     close $stderr_read if defined $stderr_read;
-    
-    # Need to close on Win32 because std handles aren't dup'ed, no
-    # harm elsewhere.  Close STDERR later to not influence possible
-    # die.
-    close STDIN;
-    close STDOUT;
+
+    # Win32 needs the stdio handles closed before they're reopened
+    # because the standard handles aren't dup'd.
 
     # Redirect STDIN from the read end of the stdin pipe.
+    close STDIN if POE::Kernel::RUNNING_IN_HELL;
     open( STDIN, "<&" . fileno($stdin_read) )
       or die "can't redirect STDIN in child pid $$: $!";
 
     # Redirect STDOUT to the write end of the stdout pipe.
     # The STDOUT_FILENO check snuck in on a patch.  I'm not sure why
     # we care what the file descriptor is.
+    close STDOUT if POE::Kernel::RUNNING_IN_HELL;
     open( STDOUT, ">&" . fileno($stdout_write) )
       or die "can't redirect stdout in child pid $$: $!";
 
-    # Need to close on Win32 because std handles aren't dup'ed, no harm elsewhere
-    close STDERR;
-
     # Redirect STDERR to the write end of the stderr pipe.  If the
     # stderr pipe's undef, then we use STDOUT.
     # The STDERR_FILENO check snuck in on a patch.  I'm not sure why
     # we care what the file descriptor is.
+    close STDERR if POE::Kernel::RUNNING_IN_HELL;
     open( STDERR, ">&" . fileno($stderr_write) )
       or die "can't redirect stderr in child: $!";
 
@@ -432,15 +437,27 @@ sub new {
     close $sem_pipe_write;
 
     if (POE::Kernel::RUNNING_IN_HELL)  {
-	# The Win32 pseudo fork sets up the std handles in the child based on the true win32 handles
-	# For the exec these get remembered, so manipulation of STDIN/OUT/ERR is not enough. Only 
-	# necessary for the exec, as Perl CODE subroutine goes through 0/1/2 which are correct. 
-	# But ofcourse that coderef might invoke exec, so better do it regardless.
-	# HACK: Using Win32::Console as nothing else exposes SetStdHandle
-	Win32::Console::_SetStdHandle(STD_INPUT_HANDLE(),  FdGetOsFHandle(fileno($stdin_read)));
-	Win32::Console::_SetStdHandle(STD_OUTPUT_HANDLE(), FdGetOsFHandle(fileno($stdout_write)));
-	Win32::Console::_SetStdHandle(STD_ERROR_HANDLE(),  FdGetOsFHandle(fileno($stderr_write)));
+      # The Win32 pseudo fork sets up the std handles in the child
+      # based on the true win32 handles For the exec these get
+      # remembered, so manipulation of STDIN/OUT/ERR is not enough.
+      # Only necessary for the exec, as Perl CODE subroutine goes
+      # through 0/1/2 which are correct.  But ofcourse that coderef
+      # might invoke exec, so better do it regardless.
+      # HACK: Using Win32::Console as nothing else exposes SetStdHandle
+      Win32::Console::_SetStdHandle(
+        STD_INPUT_HANDLE(),
+        FdGetOsFHandle(fileno($stdin_read))
+      );
+      Win32::Console::_SetStdHandle(
+        STD_OUTPUT_HANDLE(),
+        FdGetOsFHandle(fileno($stdout_write))
+      );
+      Win32::Console::_SetStdHandle(
+        STD_ERROR_HANDLE(),
+        FdGetOsFHandle(fileno($stderr_write))
+      );
     }
+
     # Exec the program depending on its form.
     if (ref($program) eq 'CODE') {
 
@@ -463,22 +480,24 @@ sub new {
 
       # Try to exit without triggering END or object destructors.
       # Give up with a plain exit if we must.
-      # On win32 cannot _exit as it will kill *all* threads, meaning parent too
+      # But we can't _exit on Win32 because it KILLS ALL THREADS,
+      # including the parent "process".
       unless (POE::Kernel::RUNNING_IN_HELL) {
-	  eval { POSIX::_exit(0);  };
-	  eval { kill KILL => $$;  };
-	  eval { exec("$^X -e 0"); };
+        eval { POSIX::_exit(0);  };
+        eval { kill KILL => $$;  };
+        eval { exec("$^X -e 0"); };
       };
       exit(0);
-    } else {
-	if (ref($program) eq 'ARRAY') {
-	  exec(@$program, @$prog_args)
-	    or die "can't exec (@$program) in child pid $$: $!";
-	}
-	else {
-	  exec(join(" ", $program, @$prog_args))
-	    or die "can't exec ($program) in child pid $$: $!";
-	}
+    }
+    else {
+      if (ref($program) eq 'ARRAY') {
+        exec(@$program, @$prog_args)
+          or die "can't exec (@$program) in child pid $$: $!";
+      }
+      else {
+        exec(join(" ", $program, @$prog_args))
+          or die "can't exec ($program) in child pid $$: $!";
+      }
     }
     die "insanity check passed";
   }
@@ -488,9 +507,9 @@ sub new {
   close $stdout_write if defined $stdout_write;
   close $stderr_write if defined $stderr_write;
 
-  my $handle_count = 0;
-  $handle_count++ if defined $stdout_read;
-  $handle_count++ if defined $stderr_read;
+  my $active_count = 0;
+  $active_count++ if $stdout_event and $stdout_read;
+  $active_count++ if $stderr_event and $stderr_read;
 
   my $self = bless [
     &POE::Wheel::allocate_wheel_id(),  # UNIQUE_ID
@@ -499,7 +518,7 @@ sub new {
     $program,       # PROGRAM
     $pid,           # CHILD_PID
     $conduit,       # CONDUIT_TYPE
-    $handle_count,  # IS_ACTIVE
+    $active_count,  # IS_ACTIVE
     $close_on_call, # CLOSE_ON_CALL
     $stdio_type,    # STDIO_TYPE
     # STDIN
@@ -524,7 +543,10 @@ sub new {
   ], $type;
 
   # Wait here while the child sets itself up.
-  <$sem_pipe_read>;
+  {
+    local $/ = "\n";
+    <$sem_pipe_read>;
+  }
   close $sem_pipe_read;
   close $sem_pipe_write;
 
@@ -605,97 +627,99 @@ sub _define_stdin_flusher {
 sub _define_stdout_reader {
   my $self = shift;
 
+  # Can't do anything if we don't have a handle.
+  return unless defined $self->[HANDLE_STDOUT];
+
+  # No event?  Unregister the handler and leave.
+  my $stdout_event  = \$self->[EVENT_STDOUT];
+  unless ($$stdout_event) {
+    $poe_kernel->select_read($self->[HANDLE_STDOUT]);
+    return;
+  }
+
+  # If any of these change, then the read state is invalidated and
+  # needs to be redefined.
+  my $unique_id     = $self->[UNIQUE_ID];
+  my $driver        = $self->[DRIVER_STDOUT];
+  my $stdout_filter = $self->[FILTER_STDOUT];
+
+  # These can change without redefining the callback since they're
+  # enclosed by reference.
+  my $is_active     = \$self->[IS_ACTIVE];
+  my $close_event   = \$self->[CLOSE_EVENT];
+  my $error_event   = \$self->[ERROR_EVENT];
+
   # Register the select-read handler for STDOUT.
-  if (defined $self->[HANDLE_STDOUT]) {
+  if (
+    $stdout_filter->can("get_one") and
+    $stdout_filter->can("get_one_start")
+  ) {
+    $poe_kernel->state(
+      $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
 
-    # If any of these change, then the read state is invalidated and
-    # needs to be redefined.
-    my $unique_id     = $self->[UNIQUE_ID];
-    my $driver        = $self->[DRIVER_STDOUT];
-    my $error_event   = \$self->[ERROR_EVENT];
-    my $close_event   = \$self->[CLOSE_EVENT];
-    my $stdout_filter = $self->[FILTER_STDOUT];
-    my $stdout_event  = \$self->[EVENT_STDOUT];
-    my $is_active     = \$self->[IS_ACTIVE];
-
-    if (
-      $stdout_filter->can("get_one") and
-      $stdout_filter->can("get_one_start")
-    ) {
-      $poe_kernel->state(
-        $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            $stdout_filter->get_one_start($raw_input);
-            while (1) {
-              my $next_rec = $stdout_filter->get_one();
-              last unless @$next_rec;
-              foreach my $cooked_input (@$next_rec) {
-                $k->call($me, $$stdout_event, $cooked_input, $unique_id);
-              }
-            }
-          }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDOUT'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
-          }
-        }
-      );
-    }
-
-    # Otherwise we can't get one.
-    else {
-      $poe_kernel->state(
-        $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            foreach my $cooked_input (@{$stdout_filter->get($raw_input)}) {
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          $stdout_filter->get_one_start($raw_input);
+          while (1) {
+            my $next_rec = $stdout_filter->get_one();
+            last unless @$next_rec;
+            foreach my $cooked_input (@$next_rec) {
               $k->call($me, $$stdout_event, $cooked_input, $unique_id);
             }
           }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDOUT'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
+        }
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDOUT'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
+  }
+
+  # Otherwise we can't get one.
+  else {
+    $poe_kernel->state(
+      $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          foreach my $cooked_input (@{$stdout_filter->get($raw_input)}) {
+            $k->call($me, $$stdout_event, $cooked_input, $unique_id);
           }
         }
-      );
-    }
-
-    # register the state's select
-    $poe_kernel->select_read($self->[HANDLE_STDOUT], $self->[STATE_STDOUT]);
+        else {
+          $$error_event and
+            $k->call(
+              $me, $$error_event,
+              'read', ($!+0), $!, $unique_id, 'STDOUT'
+            );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
   }
 
-  # Register the select-read handler for STDOUT.
-  else {
-    $poe_kernel->select_read($self->[HANDLE_STDOUT])
-      if defined $self->[HANDLE_STDOUT];
-  }
+  # register the state's select
+  $poe_kernel->select_read($self->[HANDLE_STDOUT], $self->[STATE_STDOUT]);
 }
 
 #------------------------------------------------------------------------------
@@ -705,96 +729,96 @@ sub _define_stdout_reader {
 sub _define_stderr_reader {
   my $self = shift;
 
+  # Can't do anything if we don't have a handle.
+  return unless defined $self->[HANDLE_STDERR];
+
+  # No event?  Unregister the handler and leave.
+  my $stderr_event  = \$self->[EVENT_STDERR];
+  unless ($$stderr_event) {
+    $poe_kernel->select_read($self->[HANDLE_STDERR]);
+    return;
+  }
+
+  my $unique_id     = $self->[UNIQUE_ID];
+  my $driver        = $self->[DRIVER_STDERR];
+  my $stderr_filter = $self->[FILTER_STDERR];
+
+  # These can change without redefining the callback since they're
+  # enclosed by reference.
+  my $error_event   = \$self->[ERROR_EVENT];
+  my $close_event   = \$self->[CLOSE_EVENT];
+  my $is_active     = \$self->[IS_ACTIVE];
+
   # Register the select-read handler for STDERR.
-  if (defined $self->[HANDLE_STDERR]) {
-    # If any of these change, then the read state is invalidated and
-    # needs to be redefined.
-    my $unique_id     = $self->[UNIQUE_ID];
-    my $driver        = $self->[DRIVER_STDERR];
-    my $error_event   = \$self->[ERROR_EVENT];
-    my $close_event   = \$self->[CLOSE_EVENT];
-    my $stderr_filter = $self->[FILTER_STDERR];
-    my $stderr_event  = \$self->[EVENT_STDERR];
-    my $is_active     = \$self->[IS_ACTIVE];
+  if (
+    $stderr_filter->can("get_one") and
+    $stderr_filter->can("get_one_start")
+  ) {
+    $poe_kernel->state(
+      $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
 
-    if (
-      $stderr_filter->can("get_one") and
-      $stderr_filter->can("get_one_start")
-    ) {
-      $poe_kernel->state(
-        $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            $stderr_filter->get_one_start($raw_input);
-            while (1) {
-              my $next_rec = $stderr_filter->get_one();
-              last unless @$next_rec;
-              foreach my $cooked_input (@$next_rec) {
-                $k->call($me, $$stderr_event, $cooked_input, $unique_id);
-              }
-            }
-          }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDERR'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
-          }
-        }
-      );
-    }
-
-    # Otherwise we can't get_one().
-    else {
-      $poe_kernel->state(
-        $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            foreach my $cooked_input (@{$stderr_filter->get($raw_input)}) {
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          $stderr_filter->get_one_start($raw_input);
+          while (1) {
+            my $next_rec = $stderr_filter->get_one();
+            last unless @$next_rec;
+            foreach my $cooked_input (@$next_rec) {
               $k->call($me, $$stderr_event, $cooked_input, $unique_id);
             }
           }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDERR'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
+        }
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDERR'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
+  }
+
+  # Otherwise we can't get_one().
+  else {
+    $poe_kernel->state(
+      $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          foreach my $cooked_input (@{$stderr_filter->get($raw_input)}) {
+            $k->call($me, $$stderr_event, $cooked_input, $unique_id);
           }
         }
-      );
-    }
-
-    # register the state's select
-    $poe_kernel->select_read($self->[HANDLE_STDERR], $self->[STATE_STDERR]);
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDERR'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
   }
 
-  # Register the select-read handler for STDERR.
-  else {
-    $poe_kernel->select_read($self->[HANDLE_STDERR])
-      if defined $self->[HANDLE_STDERR];
-  }
+  # Register the state's select.
+  $poe_kernel->select_read($self->[HANDLE_STDERR], $self->[STATE_STDERR]);
 }
 
 #------------------------------------------------------------------------------
@@ -837,9 +861,11 @@ sub event {
     }
   }
 
-  $self->_define_stdin_flusher() if defined $redefine_stdin;
-  $self->_define_stdout_reader() if defined $redefine_stdout;
-  $self->_define_stderr_reader() if defined $redefine_stderr;
+  # Recalculate the active handles count.
+  my $active_count = 0;
+  $active_count++ if $self->[EVENT_STDOUT] and $self->[HANDLE_STDOUT];
+  $active_count++ if $self->[EVENT_STDERR] and $self->[HANDLE_STDERR];
+  $self->[IS_ACTIVE] = $active_count;
 }
 
 #------------------------------------------------------------------------------
@@ -884,6 +910,12 @@ sub DESTROY {
 
 sub put {
   my ($self, @chunks) = @_;
+
+  # Avoid big bada boom if someone put()s on a dead wheel.
+  croak "Called put() on a wheel without an open STDIN handle" unless (
+    $self->[HANDLE_STDIN]
+  );
+
   if (
     $self->[OCTETS_STDIN] =  # assignment on purpose
     $self->[DRIVER_STDIN]->put($self->[FILTER_STDIN]->put(\@chunks))
@@ -930,7 +962,11 @@ sub shutdown_stdin {
   $poe_kernel->select_write($self->[HANDLE_STDIN], undef);
 
   eval { local $^W = 0; shutdown($self->[HANDLE_STDIN], 1) };
-  close $self->[HANDLE_STDIN] if $@;
+  if ($@ or $self->[HANDLE_STDIN] != $self->[HANDLE_STDOUT]) {
+    close $self->[HANDLE_STDIN];
+  }
+
+  $self->[HANDLE_STDIN] = undef;
 }
 
 #------------------------------------------------------------------------------
@@ -1040,7 +1076,6 @@ sub set_stdout_filter {
   my $buf = $self->[FILTER_STDOUT]->get_pending();
   $self->[FILTER_STDOUT] = $new_filter;
 
-  $self->_define_stdout_reader();
   $self->_transfer_stdout_buffer($buf);
 }
 
@@ -1050,7 +1085,6 @@ sub set_stderr_filter {
   my $buf = $self->[FILTER_STDERR]->get_pending();
   $self->[FILTER_STDERR] = $new_filter;
 
-  $self->_define_stderr_reader();
   $self->_transfer_stderr_buffer($buf);
 }
 
@@ -1110,35 +1144,33 @@ POE::Wheel::Run - event driven fork/exec with added value
   $program = [ '/usr/bin/cat', '-' ];
 
   $wheel = POE::Wheel::Run->new(
+    # Set the program to execute, and optionally some parameters.
     Program     => $program,
-    ProgramArgs => \@program_args,     # Parameters for $program.
-    Priority    => +5,                 # Adjust priority.  May need to be root.
-    User        => getpwnam('nobody'), # Adjust UID. May need to be root.
-    Group       => getgrnam('nobody'), # Adjust GID. May need to be root.
-    ErrorEvent  => 'oops',             # Event to emit on errors.
-    CloseEvent  => 'child_closed',     # Child closed all output.
+    ProgramArgs => \@program_args,
 
-    StdinEvent  => 'stdin',  # Event to emit when stdin is flushed to child.
-    StdoutEvent => 'stdout', # Event to emit with child stdout information.
-    StderrEvent => 'stderr', # Event to emit with child stderr information.
+    # Define I/O events to emit.  Most are optional.
+    StdinEvent  => 'stdin',     # Flushed all data to the child's STDIN.
+    StdoutEvent => 'stdout',    # Received data from the child's STDOUT.
+    StderrEvent => 'stderr',    # Received data from the child's STDERR.
+    ErrorEvent  => 'oops',          # An I/O error occurred.
+    CloseEvent  => 'child_closed',  # Child closed all output handles.
 
-    # Specify different I/O formats.
+    # Optionally adjust the child process priority, user ID, and/or
+    # group ID.  You may need to be root to do this.
+    Priority    => +5,
+    User        => scalar(getpwnam 'nobody'),
+    Group       => scalar(getgrnam 'nobody'),
+
+    # Optionally specify different I/O formats.
     StdinFilter  => POE::Filter::Line->new(),   # Child accepts input as lines.
     StdoutFilter => POE::Filter::Stream->new(), # Child output is a stream.
     StderrFilter => POE::Filter::Line->new(),   # Child errors are lines.
 
-    # Set StdinFilter and StdoutFilter together.
+    # Shorthand to set StdinFilter and StdoutFilter together.
     StdioFilter => POE::Filter::Line->new(),    # Or some other filter.
-
-    # Specify different I/O methods.
-    StdinDriver  => POE::Driver::SysRW->new(),  # Defaults to SysRW.
-    StdoutDriver => POE::Driver::SysRW->new(),  # Same.
-    StderrDriver => POE::Driver::SysRW->new(),  # Same.
-
-    # Set StdinDriver and StdoutDriver together.
-    StdioDriver  => POE::Driver::SysRW->new(),
   );
 
+  # Information about the wheel and its process.
   print "Unique wheel ID is  : ", $wheel->ID;
   print "Wheel's child PID is: ", $wheel->PID;
 
@@ -1146,14 +1178,27 @@ POE::Wheel::Run - event driven fork/exec with added value
   $wheel->put( 'input for the child' );
 
   # Kill the child.
-  $wheel->kill();  # TERM by default
-  $wheel->kill(9);
+  $wheel->kill(9);  # TERM by default.
 
 =head1 DESCRIPTION
 
 Wheel::Run spawns child processes and establishes non-blocking, event
 based communication with them.
 
+Wheel::Run does not reap child processes.  For that, you need to
+register a SIGCHLD handler:
+
+  $kernel->sig(CHLD => "your_event");
+
+The session will then receive your_event with details about $? when
+the wheel's process exits and is reaped.  POE will reap child
+processes as a side effect.
+
+Another way to do it is to register $SIG{CHLD} = "IGNORE".  Use
+sparingly and with caution: This may clobber a handler that POE has
+already registered for SIGCHLD.  Why does IGNORE work this way?  See
+the discussion in perldoc perlipc.
+
 =head1 PUBLIC METHODS
 
 =over 2
@@ -1175,7 +1220,9 @@ process.  By default it will try various forms of inter-process
 communication to build a pipe between the parent and child processes.
 If a particular method is preferred, it can be set to "pipe",
 "socketpair", or "inet".  It may also be set to "pty" if the child
-process should have its own pseudo tty.
+process should have its own pseudo tty.  Setting it to "pty-pipe"
+gives the child process a stdin and stdout pseudo-tty, but keeps
+stderr as a pipe, rather than merging stdout and stderr as with "pty".
 
 The reasons to define this parameter would be if you want to use
 "pty", if the default pipe type doesn't work properly on your
@@ -1233,6 +1280,9 @@ same time.
 
 =item StderrEvent
 
+See L<EVENTS AND PARAMETERS> below for a more detailed description of
+these events and their parameters.
+
 C<CloseEvent> contains the name of an event to emit when the child
 process closes all its output handles.  This is a consistent
 notification that the child will not be sending any more output.  It
@@ -1241,17 +1291,13 @@ accepting input.
 
 C<ErrorEvent> contains the name of an event to emit if something
 fails.  It is optional and if omitted, the wheel will not notify its
-session if any errors occur.  The event receives 5 parameters as
-follows: ARG0 = the return value of syscall(), ARG1 = errno() - the
-numeric value of the error generated, ARG2 = error() - a descriptive
-for the given error, ARG3 = the wheel id, and ARG4 = the handle on
-which the error occurred (stdout, stderr, etc.)
+session if any errors occur.
 
 Wheel::Run requires at least one of the following three events:
 
 C<StdinEvent> contains the name of an event that Wheel::Run emits
-whenever all its output has been flushed to the child process' STDIN
-handle.
+whenever everything queued by its put() method has been flushed to the
+child's STDIN handle.
 
 C<StdoutEvent> and C<StderrEvent> contain names of events that
 Wheel::Run emits whenever the child process writes something to its
@@ -1410,6 +1456,12 @@ tend to be reluctant to exit when their terminals are closed.
 
 The kill() method will send SIGTERM if SIGNAL is undef or omitted.
 
+=item get_driver_out_messages
+
+=item get_driver_out_octets
+
+Return driver statistics.
+
 =back
 
 =head1 EVENTS AND PARAMETERS
@@ -1443,11 +1495,13 @@ whenever an error occurs.  Every error event comes with four
 parameters:
 
 C<ARG0> contains the name of the operation that failed.  It may be
-'read' or 'write' or 'fork' or 'exec' or something.  The actual values
-aren't yet defined.  Note: This is not necessarily a function name.
+'read', 'write', 'fork', 'exec' or the name of some other function or
+task.  The actual values aren't yet defined.  Note: This is not
+necessarily a function name.
 
 C<ARG1> and C<ARG2> hold numeric and string values for C<$!>,
-respectively.
+respectively.  C<"$!"> will eq C<""> for read error 0 (child process
+closed STDOUT or STDERR).
 
 C<ARG3> contains the wheel's unique ID.
 
@@ -1461,6 +1515,7 @@ A sample error event handler:
 
   sub error_state {
     my ($operation, $errnum, $errstr, $wheel_id) = @_[ARG0..ARG3];
+    $errstr = "remote end closed" if $operation eq "read" and !$errnum;
     warn "Wheel $wheel_id generated $operation error $errnum: $errstr\n";
   }
 
@@ -1477,9 +1532,9 @@ StdinEvent's C<ARG0> parameter contains its wheel's unique ID.
 =item StderrEvent
 
 StdoutEvent and StderrEvent contain names for events that Wheel::Run
-emits whenever the child process makes output.  StdoutEvent contains
-information the child wrote to its STDOUT handle, and StderrEvent
-includes whatever arrived from the child's STDERR handle.
+emits whenever the child process generates new output.  StdoutEvent
+contains information the child wrote to its STDOUT handle, and
+StderrEvent includes whatever arrived from the child's STDERR handle.
 
 Both of these events come with two parameters.  C<ARG0> contains the
 information that the child wrote.  C<ARG1> holds the wheel's unique
