@@ -1,16 +1,17 @@
-# $Id: SocketFactory.pm 1903 2006-03-20 04:44:08Z rcaputo $
+# $Id: SocketFactory.pm 2194 2007-07-21 06:51:26Z rcaputo $
 
 package POE::Wheel::SocketFactory;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2194 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw( carp croak );
 use Symbol qw( gensym );
 
 use POSIX qw(:fcntl_h);
+use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
 use Errno qw(EWOULDBLOCK EADDRNOTAVAIL EINPROGRESS EADDRINUSE);
 use Socket qw(
   AF_INET SOCK_STREAM SOL_SOCKET AF_UNIX PF_UNIX 
@@ -19,6 +20,7 @@ use Socket qw(
   pack_sockaddr_in pack_sockaddr_un inet_aton SOMAXCONN
 );
 use IO::Handle ();
+use FileHandle ();
 use POE qw( Wheel );
 
 sub CRIMSON_SCOPE_HACK ($) { 0 }
@@ -44,11 +46,21 @@ sub MY_SOCKET_SELECTED () { 12 }
 # know you've broken his module.
 
 # Provide dummy constants for systems that don't have them.
+# Test and provide for each constant separately, per suggestion in
+# rt.cpan.org 27250.
 BEGIN {
-  unless (exists $INC{"Socket6.pm"}) {
-    eval "*Socket6::AF_INET6 = sub () { ~0 }";
-    eval "*Socket6::PF_INET6 = sub () { ~0 }";
-  }
+  eval {
+    require Socket6;
+    my $x = &Socket6::AF_INET6;
+  };
+  *Socket6::AF_INET6 = sub () { ~0 } if $@;
+
+  eval {
+    require Socket6;
+    my $x = &Socket6::PF_INET6;
+  };
+
+  *Socket6::PF_INET6 = sub () { ~0 } if $@;
 }
 
 #------------------------------------------------------------------------------
@@ -107,7 +119,7 @@ my %default_socket_type = (
 # Perform system-dependent translations on Unix addresses, if
 # necessary.
 
-sub condition_unix_address {
+sub _condition_unix_address {
   my ($address) = @_;
 
   # OS/2 would like sockets to use backwhacks, and please place them
@@ -713,7 +725,7 @@ sub new {
       );
 
       # Need to check lengths in octets, not characters.
-      use bytes;
+      BEGIN { eval { require bytes } and bytes->import; }
 
       # Resolve the bind address if it's not already packed.
       unless (length($bind_address) == 4) {
@@ -786,7 +798,7 @@ sub new {
       }
 
       # Need to check lengths in octets, not characters.
-      use bytes;
+      BEGIN { eval { require bytes } and bytes->import; }
 
       # Resolve the bind address.
       my @info = Socket6::getaddrinfo(
@@ -823,7 +835,7 @@ sub new {
         return $self;
       }
 
-      $bind_address = &condition_unix_address($params{BindAddress});
+      $bind_address = &_condition_unix_address($params{BindAddress});
       $bind_address = pack_sockaddr_un($bind_address);
       unless ($bind_address) {
         $poe_kernel->yield(
@@ -950,7 +962,7 @@ sub new {
     # understands.
     elsif ($abstract_domain eq DOM_UNIX) {
 
-      $connect_address = condition_unix_address($params{RemoteAddress});
+      $connect_address = _condition_unix_address($params{RemoteAddress});
       $connect_address = pack_sockaddr_un($connect_address);
       unless (defined $connect_address) {
         $poe_kernel->yield(
